%option stack yymore noyywrap full ecs

%{
/* This is a parameter to xmlbase. */
#define YYLEX_OUT 

#include "xmlbase.h"
#include "y.tab.h"

const char* token_names[] = {
       NULL,
       "SCDECL",
       "XSCDECL",
       "SECTEND",
       "BOL",
       "LBRACE",
       "RBRACE",
       "EOL",
       "EOF_OP",
       "EOF_TOK",
       "FALLTHRU_ACT",
       "ASSIGN_ACT",
       "FOLLOWED_BY",
       "ALTERNATIVE",
       "STAR",
       "PLUS",
       "OPT",
       "DOT",
       "LPAREN",
       "RPAREN",
       "NOT",
       "CCE_ALNUM",
       "CCE_ALPHA",
       "CCE_BLANK",
       "CCE_CNTRL",
       "CCE_DIGIT",
       "CCE_GRAPH",
       "CCE_LOWER",
       "CCE_PRINT",
       "CCE_PUNCT",
       "CCE_SPACE",
       "CCE_UPPER",
       "CCE_XDIGIT",
       "LBRACK",
       "RBRACK",
       "LANGLE",
       "RANGLE",
       "TOKENS",
       "COUNTERS",
       "SYMTBLS",
       "NL",
       "DASH",
       "WS",
       "TOKEN_ACT",
       "TOKENTEXT_ACT",
       "IF_ACT",
       "THEN_ACT",
       "ELSE_ACT",
       "PUSH_ACT",
       "POP_ACT",
       "BEGIN_ACT",
       "START_ACT",
       "END_ACT",
       "ECHO_ACT",
       "SKIP_ACT",
       "ERROR_ACT",
       "INCR_ACT",
       "DECR_ACT",
       "ADD_ACT",
       "REMOVE_ACT",
       "CONTINUE_ACT",
	"CUT_ACT",
	"TRIM_ACT",
       "SEMI",
       "COMMA",
       "EQUALS",
       "NEQUALS",
       "LT",
       "LTE",
       "GT",
       "GTE",
       "ID",
       "MEM",
       "ABBREV_ID",
       "ESCAPE",
       "CHAR",
       "NUMBER",
       "REF",
       "QUOTED",
       "STRING"
     };

int CNTR_lineno = 0;
       
%}

%s SECT2 CODEBLOCK PICKUPDEF SC CARETISBOL NUM QUOTE DIDADEF
%s FIRSTCCL CCL ACTION RECOVER COMMENT REGEXP REFERENCE EXIT

WS		[[:blank:]]+
OPTWS		[[:blank:]]*
NOT_WS		[^[:blank:]\n]

NL		\r?\n

NAME		([[:alpha:]_][[:alnum:]_-]*)
NOT_NAME	[^[:alpha:]_*\n]+

SCNAME		{NAME}

ESCSEQ		(\\([^\n]|[0-7]{1,3}|x[[:xdigit:]]{1,2}))

FIRST_CCL_CHAR	([^\\\n]|{ESCSEQ})
CCL_CHAR	([^\\\n\]]|{ESCSEQ})
CCL_EXPR	("[:"[[:alpha:]]+":]")


%%
	static int bracelevel;

<INITIAL>{
	^"/*"		{PUSH( COMMENT );}
        ^"%s"      	{TOKEN(SCDECL);}
        ^"%x"           {TOKEN(XSCDECL);}
        ^"%token"       {TOKEN(TOKENS);}
        ^"%counter"     {TOKEN(COUNTERS);}
        ^"%symtbl"      {TOKEN(SYMTBLS);}
	{WS}		/* discard */

	^"%%".*		{
			bracelevel = 0;
			BEGIN(SECT2);
			TOKEN(SECTEND);
			}

	^"%"[^csxt].*	        {ERROR("unrecognized '%' directive");}

	^{NAME}		{
			BEGIN(PICKUPDEF);
			TOKENTEXT(ABBREV_ID);
			}

	{SCNAME}	{TOKENTEXT(ID);}
	^{OPTWS}{NL}	{INCR(CNTR_lineno);TOKEN(NL);} /* allows blank lines in section 1 */
	{OPTWS}{NL}	{INCR(CNTR_lineno);TOKEN(NL);} /* maybe end of comment line */
}



<COMMENT>{
	"*/"		{POP;}
	"*"		{}
	[^*\n]+		{}
	[^*\n]*{NL}	{INCR(CNTR_lineno);}
}

<PICKUPDEF>{
	{WS}/{NOT_WS}  {
	                BEGIN(DIDADEF);PUSH(REGEXP);TOKEN(WS);
			}

	{NL}		{
                        ERROR("incomplete name definition" );
			}
}
<DIDADEF>{
        .*	SKIP;

	{NL}		{
			BEGIN(INITIAL);
			INCR(CNTR_lineno);
                        TOKEN(NL);
			}
}



<RECOVER>.*{NL}		{INCR(CNTR_lineno); BEGIN(INITIAL);TOKEN(NL);}

<SECT2>{
	^{OPTWS}{NL}	INCR(CNTR_lineno); /* allow blank lines in section 2 */

	^{OPTWS}"<"	BEGIN(SC);TOKEN(LANGLE);
        ^{OPTWS}"^"	TOKEN(BOL);
	\"		PUSH(QUOTE); START(QUOTED);
	"{"/[[:digit:]]	PUSH(NUM); TOKEN(LBRACE);
        "{"/{NAME}      PUSH(REFERENCE);
	"$"/([[:blank:]]|{NL})	TOKEN(EOL);

	^{WS}"/*"	{ /* what the hell does this do? */
			yyless( yyleng - 2 );	/* put back '/', '*' */
			bracelevel = 0;
			BEGIN(ACTION);
                        TOKEN(WS);
			}

	^{WS}		/* allow indented rules */

	{WS}		{
			bracelevel = 0;
			BEGIN(ACTION);
                        TOKEN(WS);
			}

	{OPTWS}{NL}	{
                        INCR(CNTR_lineno); 
                        TOKEN(NL);
			}

	^{OPTWS}"<<EOF>>"	|
	"<<EOF>>"	TOKEN(EOF_OP);

	"["            PUSH(FIRSTCCL);TOKEN(LBRACK); 

	[/]            TOKEN(FOLLOWED_BY);
        [|]            TOKEN(ALTERNATIVE);
        [*]            TOKEN(STAR);
        [+]            TOKEN(PLUS);
        [?]            TOKEN(OPT);
        [.]            TOKEN(DOT);
        [(]            TOKEN(LPAREN);
        [)]            TOKEN(RPAREN);
        [{]            TOKEN(LBRACE);
        [}]            TOKEN(RBRACE);

	.	       TOKENTEXT(CHAR);
        <<EOF>>        BEGIN(EXIT);TOKEN(EOF_TOK);
}

<EXIT>{
        <<EOF>>        ERROR("unexpected EOF");
}

<REGEXP>{

	\"		PUSH(QUOTE); START(QUOTED);
	"{"/[[:digit:]]	PUSH(NUM); TOKEN(LBRACE);
	"{"/{NAME}	PUSH(REFERENCE);

        {WS}            {POP;}
	{NL}            {POP;BEGIN(INITIAL);INCR(CNTR_lineno);TOKEN(NL);}
	"<<EOF>>"	TOKEN(EOF_OP);

	"["            PUSH(FIRSTCCL);TOKEN(LBRACK); 

	[/]            TOKEN(FOLLOWED_BY);
        [|]            TOKEN(ALTERNATIVE);
        [*]            TOKEN(STAR);
        [+]            TOKEN(PLUS);
        [?]            TOKEN(OPT);
        [.]            TOKEN(DOT);
        [(]            TOKEN(LPAREN);
        [)]            TOKEN(RPAREN);
        [{]            TOKEN(LBRACE);
        [}]            TOKEN(RBRACE);


	.	       TOKENTEXT(CHAR);
}

<REFERENCE>{
  {NAME}          TOKENTEXT(REF);
  [}]             POP;
}


<SC>{
	[,]             TOKEN(COMMA);
        [*]		TOKEN(STAR);
	">"		BEGIN(SECT2);TOKEN(RANGLE);
	">"/^		BEGIN(CARETISBOL);TOKEN(RANGLE);
	{SCNAME}	TOKENTEXT(ID);
	.		ERROR( "bad <start condition>");
}

<CARETISBOL>"^"		{BEGIN(SECT2); TOKEN(BOL);}


<QUOTE>{
	[^"\n]		{CONTINUE;}
	\"		{POP;TRIM(1);END;}
        \\\"            {CONTINUE;}
	{NL}		{
			ERROR( "missing quote" );
			}
}


<FIRSTCCL>{
	"^"/[^-\]\n]	BEGIN(CCL); TOKEN(NOT); 
	"^"/("-"|"]")   TOKENTEXT(CHAR);
	.		BEGIN(CCL); TOKENTEXT(CHAR);
}

<CCL>{
	-/[^\]\n]	TOKEN(DASH);
	[^\]\n]		TOKENTEXT(CHAR);
	"]"		POP;TOKEN(RBRACK);
	.|{NL}		{
			ERROR( "bad character class" );
			}
}

<FIRSTCCL,CCL>{
	"[:alnum:]"	BEGIN(CCL); TOKEN(CCE_ALNUM);
	"[:alpha:]"	BEGIN(CCL); TOKEN(CCE_ALPHA);
	"[:blank:]"	BEGIN(CCL); TOKEN(CCE_BLANK);
	"[:cntrl:]"	BEGIN(CCL); TOKEN(CCE_CNTRL);
	"[:digit:]"	BEGIN(CCL); TOKEN(CCE_DIGIT);
	"[:graph:]"	BEGIN(CCL); TOKEN(CCE_GRAPH);
	"[:lower:]"	BEGIN(CCL); TOKEN(CCE_LOWER);
	"[:print:]"	BEGIN(CCL); TOKEN(CCE_PRINT);
	"[:punct:]"	BEGIN(CCL); TOKEN(CCE_PUNCT);
	"[:space:]"	BEGIN(CCL); TOKEN(CCE_SPACE);
	"[:upper:]"	BEGIN(CCL); TOKEN(CCE_UPPER);
	"[:xdigit:]"	BEGIN(CCL); TOKEN(CCE_XDIGIT);
	{CCL_EXPR}	{
			ERROR("bad character class expression:" );
			}
}

<NUM>{
	[[:digit:]]+	{
			TOKENTEXT(NUMBER);
			}

	","		TOKEN(COMMA);
	"}"		POP; TOKEN(RBRACE);

	.		{
			ERROR( "bad character inside {}'s" );
			}

	{NL}		{
			ERROR( "missing }" );
			}
}



<ACTION>{
	"{"		++bracelevel; TOKEN(LBRACE);
	"}"		--bracelevel; TOKEN(RBRACE);
        "("             TOKEN(LPAREN);
        ")"             TOKEN(RPAREN);
        "|"             TOKEN(FALLTHRU_ACT);
	"="		TOKEN(ASSIGN_ACT);
        "TOKEN"         TOKEN(TOKEN_ACT);
        "TOKENTEXT"     TOKEN(TOKENTEXT_ACT);
        "IF"            TOKEN(IF_ACT);
        "THEN"          TOKEN(THEN_ACT); 
        "ELSE"          TOKEN(ELSE_ACT);
        "PUSH"          TOKEN(PUSH_ACT);
        "POP"           TOKEN(POP_ACT);
        "BEGIN"         TOKEN(BEGIN_ACT);
        "START"         TOKEN(START_ACT);
        "END"           TOKEN(END_ACT);
        "ECHO"          TOKEN(ECHO_ACT);
        "SKIP"          TOKEN(SKIP_ACT);
        "ERROR"         TOKEN(ERROR_ACT);
        "INCR"          TOKEN(INCR_ACT);
	"DECR"          TOKEN(DECR_ACT);
        "ADD"           TOKEN(ADD_ACT);
        "REMOVE"        TOKEN(REMOVE_ACT);
	"CUT"		TOKEN(CUT_ACT);
	"TRIM"		TOKEN(TRIM_ACT);
        "CONTINUE"      TOKEN(CONTINUE_ACT);
        "=="            TOKEN(EQUALS);
	"!="            TOKEN(NEQUALS);
	"<"             TOKEN(LT);
	"<="            TOKEN(LTE);
	">"             TOKEN(GT);
	">="            TOKEN(GTE);
        "MEMBER"        TOKEN(MEM);
        {NAME}          TOKENTEXT(ID);
	\"              PUSH(QUOTE); START(STRING);
	[[:digit:]]+    TOKENTEXT(NUMBER);
	{NL}		{
			INCR(CNTR_lineno);
			if ( bracelevel == 0 )
				{
				BEGIN(SECT2);
                                TOKEN(NL);
				}
			}
        {WS}            SKIP;
	[;]               TOKEN(SEMI);
      
}

<COMMENT,ACTION><<EOF>>	{
			ERROR( "EOF encountered inside an action or comment");
			}

<SECT2,QUOTE,FIRSTCCL,CCL>{ESCSEQ}	{
			if ( YY_START == FIRSTCCL )
				BEGIN(CCL);
                        TOKENTEXT(ESCAPE);
			}


%%