<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text"/>

  <xsl:include href="regexp.xsl"/>


<!-- General definitions -->

  <xsl:strip-space elements="*"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

<!-- Toplevel -->

  <xsl:template match="/">
    <xsl:apply-templates select="xlex"/>
  </xsl:template>

  <xsl:template match="xlex">
    <xsl:apply-templates select="declarations"/>
    <xsl:value-of select="$newline"/>
    <xsl:text>%%</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:apply-templates select="rules"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>


<!-- Declarations -->

  <xsl:template match="declarations">
    <xsl:apply-templates select="inclusive_states|exclusive_states|token_declaration|abbreviation|counter_declaration|symtbl_declaration"/>
  </xsl:template>

  <xsl:template match="inclusive_states">
    <xsl:text>%s</xsl:text>
    <xsl:for-each select="state_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="exclusive_states">
    <xsl:text>%s</xsl:text>
    <xsl:for-each select="state_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/> 
  </xsl:template>

  <xsl:template match="abbreviation">
    <xsl:value-of select="id"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="regexp" />
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="token_declaration">
    <xsl:text>%token</xsl:text>
    <xsl:for-each select="token_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/> 
  </xsl:template>

  <xsl:template match="counter_declaration">
    <xsl:text>%counter</xsl:text>
    <xsl:for-each select="counter_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/> 
  </xsl:template>

  <xsl:template match="symtbl_declaration">
    <xsl:text>%symtbl</xsl:text>
    <xsl:for-each select="symtbl_id">
      <xsl:text> </xsl:text>
      <xsl:value-of select="."/>
    </xsl:for-each>
    <xsl:value-of select="$newline"/> 
  </xsl:template>

<!-- Rules (inherit rules for regexps) -->


  <xsl:template match="rules">
    <xsl:apply-templates select="rule|scope"/>
  </xsl:template>


  <xsl:template match="rule">
    <xsl:apply-templates select="scon"/>
    <xsl:apply-templates select="regexp"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="action"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>



  <xsl:template match="scon">
    <xsl:text>&lt;</xsl:text>
    <xsl:for-each select="state_id">
      <xsl:value-of select="."/>
      <xsl:if test="position() != last()">
        <xsl:text>,</xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>&gt;</xsl:text>
  </xsl:template>

  <xsl:template match="any_state">
    <xsl:text>*</xsl:text>
  </xsl:template>

  <xsl:template match="scope">
    <xsl:apply-templates select="scon"/>
    <xsl:text>{</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:apply-templates select="rule|scope"/>
    <xsl:text>}</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <xsl:template match="action|then|else">
    <xsl:text></xsl:text>
    <xsl:apply-templates select="./*"/>
    <xsl:text></xsl:text>
  </xsl:template>

  <xsl:template match="fallthru">
    <xsl:text>|</xsl:text>
  </xsl:template>


  <xsl:template match="token">
    <xsl:text>TOKEN(</xsl:text>
    <xsl:value-of select="token_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="tokentext">
    <xsl:text>TOKENTEXT(</xsl:text>
    <xsl:value-of select="token_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="assign">
    <xsl:value-of select="counter_id"/>
    <xsl:text> := </xsl:text>
    <xsl:value-of select="num"/>
    <xsl:text>;</xsl:text>
  </xsl:template>

  <xsl:template match="begin">
    <xsl:text>BEGIN(</xsl:text>
    <xsl:value-of select="state_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="push">
    <xsl:text>PUSH(</xsl:text>
    <xsl:value-of select="state_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="pop">
    <xsl:text>POP;</xsl:text>
  </xsl:template>


  <xsl:template match="start">
    <xsl:text>START(</xsl:text>
    <xsl:value-of select="token_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="end">
    <xsl:text>END;</xsl:text>
  </xsl:template>

  <xsl:template match="continue">
    <xsl:text>CONTINUE;</xsl:text>
  </xsl:template>

  <xsl:template match="echo">
    <xsl:text>ECHO;</xsl:text>
  </xsl:template>

  <xsl:template match="skip">
    <xsl:text>SKIP;</xsl:text>
  </xsl:template>

  <xsl:template match="cut">
    <xsl:text>CUT(</xsl:text>
      <xsl:apply-templates select="."/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="trim">
    <xsl:text>TRIM(</xsl:text>
      <xsl:apply-templates select="."/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="incr">
    <xsl:text>INCR(</xsl:text>
      <xsl:value-of select="."/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="decr">
    <xsl:text>DECR(</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="add">
    <xsl:text>ADD(</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="remove">
    <xsl:text>REMOVE(</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="error">
    <xsl:text>ERROR("</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>");</xsl:text>
  </xsl:template>

  <xsl:template match="if">
    <xsl:text>IF(</xsl:text>
    <xsl:apply-templates select="test"/>
    <xsl:text>) THEN </xsl:text>
    <xsl:apply-templates select="then"/>
    <xsl:text> ELSE </xsl:text>
    <xsl:apply-templates select="else"/>
  </xsl:template>
 
  <xsl:template match="regexp|test|exp">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="equals">
    <xsl:apply-templates select="*[1]"/>
    <xsl:text> == </xsl:text>
    <xsl:apply-templates select="*[2]"/>
  </xsl:template>

  <xsl:template match="lt">
    <xsl:apply-templates select="*[1]"/>
    <xsl:text> &lt; </xsl:text>
    <xsl:apply-templates select="*[2]"/>
  </xsl:template>

  <xsl:template match="lte">
    <xsl:apply-templates select="*[1]"/>
    <xsl:text> &lt;= </xsl:text>
    <xsl:apply-templates select="*[2]"/>
  </xsl:template>

  <xsl:template match="nequals">
    <xsl:apply-templates select="*[1]"/>
    <xsl:text> != </xsl:text>
    <xsl:apply-templates select="*[2]"/>
  </xsl:template>

  <xsl:template match="gt">
    <xsl:apply-templates select="*[1]"/>
    <xsl:text> &gt; </xsl:text>
    <xsl:apply-templates select="*[2]"/>
  </xsl:template>

  <xsl:template match="gte">
    <xsl:apply-templates select="*[1]"/>
    <xsl:text> &gt;= </xsl:text>
    <xsl:apply-templates select="*[2]"/>
  </xsl:template>

  <xsl:template match="mem">
    <xsl:text>MEMBER(</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="counter_id">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="symtbl_id">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="num">
    <xsl:value-of select="."/>
  </xsl:template>



</xsl:stylesheet>
