<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="xml" indent="yes"/>

  <xsl:strip-space elements="*"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

  <xsl:template match="/">
    <xsl:apply-templates select="xlex"/>
  </xsl:template>

  <xsl:template match="xlex">
    <xsl:element name="lex">
      <xsl:element name="declarations">
        <xsl:element name="option">stack yymore noyywrap full ecs</xsl:element>
        <xsl:element name="code_declaration">
<xsl:text>
#define YYLEX_OUT
#include "xmlbase.h"
#include "y.tab.h"
</xsl:text>
	</xsl:element>
      <xsl:call-template name="token_name_definition"/>
      <xsl:call-template name="counter_definitions"/>
      <xsl:call-template name="symtbl_definitions"/>
      <xsl:apply-templates select="declarations"/>
    </xsl:element>
    <xsl:element name="rules">
      <xsl:apply-templates select="rules"/>
    </xsl:element>
    <xsl:element name="definitions">
    </xsl:element>
    </xsl:element>
  </xsl:template>

  <xsl:template match="declarations">
    <xsl:apply-templates select="./*"/>
  </xsl:template>

  <xsl:template match="inclusive_states|exclusive_states|abbreviation">
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <xsl:template match="token_declaration|counter_declaration"> 
    <!-- Ignore, we coalesced above already -->
  </xsl:template>

  <xsl:template name="counter_definitions">
    <xsl:element name="code_declaration">
      <xsl:for-each select="/xlex/declarations/counter_declaration/counter_id">
        <xsl:text>int </xsl:text>
        <xsl:apply-templates select="."/>
	<xsl:text> = 0;</xsl:text>
	<xsl:value-of select="$newline"/>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>

  <xsl:template name="symtbl_definitions">
    <xsl:element name="code_declaration">
      <xsl:for-each select="/xlex/declarations/symtbl_declaration/symtbl_id">
        <xsl:text>struct symtbl </xsl:text>
        <xsl:apply-templates select="."/>_struct
	<xsl:text> = {0,0,NULL};</xsl:text>
	<xsl:value-of select="$newline"/>
        <xsl:text>symtbl_t </xsl:text>
        <xsl:apply-templates select="."/>
	<xsl:text> = &amp;</xsl:text>
        <xsl:apply-templates select="."/>_struct
	<xsl:text>;</xsl:text>
	<xsl:value-of select="$newline"/>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>

  <xsl:template name="token_name_definition">
    <xsl:element name="code_declaration">
      <xsl:text>
const char* token_names[] = {NULL,</xsl:text>
      <xsl:for-each select="/xlex/declarations/token_declaration/token_id">
        <xsl:text>"</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>"</xsl:text>
        <xsl:if test="position() != last()">
          <xsl:text>, </xsl:text>
        </xsl:if>
      </xsl:for-each>
<xsl:text>};
</xsl:text>
    </xsl:element>
  </xsl:template>

  <xsl:template match="rules">
    <xsl:apply-templates select="rule|scope"/>
  </xsl:template>

  <xsl:template match="rule">
    <xsl:copy>
      <xsl:copy-of select="scon"/>
      <xsl:copy-of select="regexp"/>
      <xsl:element name="action">
        <xsl:apply-templates select="action"/>
      </xsl:element>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="scope">
    <xsl:copy>
      <xsl:copy-of select="scon"/>
      <xsl:apply-templates select="rule|scope"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="then|else">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="action">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="fallthru">
    <xsl:element name="fallthru"/>
  </xsl:template>

  <xsl:template match="action_code">
    <xsl:element name="ccode">
    <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>


  <xsl:template match="assign">
    <xsl:apply-templates select="counter_id"/>
    <xsl:text> = </xsl:text>
    <xsl:value-of select="num"/>
    <xsl:text>;</xsl:text>
  </xsl:template>

  <xsl:template match="begin">
    <xsl:text>BEGIN(</xsl:text>
    <xsl:value-of select="state_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="push">
    <xsl:text>PUSH(</xsl:text>
    <xsl:value-of select="state_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="pop">
    <xsl:text>POP;</xsl:text>
  </xsl:template>

  <xsl:template match="token">
    <xsl:text>TOKEN(</xsl:text>
    <xsl:apply-templates select="token_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="tokentext">
    <xsl:text>TOKENTEXT(</xsl:text>
    <xsl:apply-templates select="token_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="start">
    <xsl:text>START(</xsl:text>
    <xsl:apply-templates select="token_id"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="continue">
    <xsl:text>CONTINUE;</xsl:text>
  </xsl:template>

  <xsl:template match="end">
    <xsl:text>END;</xsl:text>
  </xsl:template>

  <xsl:template match="skip">
    <!-- Do nothing -->
  </xsl:template>

  <xsl:template match="echo">
    <xsl:text>ECHO;</xsl:text>
  </xsl:template>

  <xsl:template match="cut">
    <xsl:text>yytext += </xsl:text>
    <xsl:apply-templates select="num"/>
    <xsl:text>;</xsl:text>
  </xsl:template>

  <xsl:template match="trim">
    <xsl:text>yytext[yyleng-</xsl:text>
    <xsl:apply-templates select="num"/>
    <xsl:text>] = '\0';</xsl:text>
  </xsl:template>

  <xsl:template match="incr">
    <xsl:apply-templates select="counter_id"/>
    <xsl:text>++;</xsl:text>
  </xsl:template>

  <xsl:template match="decr">
    <xsl:apply-templates select="counter_id"/>
    <xsl:text>--;</xsl:text>
  </xsl:template>

  <xsl:template match="add">
    <xsl:text>ADD(</xsl:text>
    <xsl:apply-templates select="symtbl_id"/>
    <xsl:text>,yytext);</xsl:text>
  </xsl:template>

  <xsl:template match="remove">
    <xsl:text>REMOVE(</xsl:text>
    <xsl:apply-templates select="symtbl_id"/>
    <xsl:text>,yytext);</xsl:text>
  </xsl:template>

  <xsl:template match="error">
    <xsl:text>ERROR("</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>");</xsl:text>
  </xsl:template>

  <xsl:template match="token_id">
    <xsl:text>TOK_</xsl:text>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="counter_id">
    <xsl:text>CNTR_</xsl:text>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="symtbl_id">
    <xsl:text>SYMTBL_</xsl:text>
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="if">
    <xsl:text>if(</xsl:text>
    <xsl:apply-templates select="test"/>
    <xsl:text>) {</xsl:text>
    <xsl:apply-templates select="then"/>
    <xsl:text>} else {</xsl:text>
    <xsl:apply-templates select="else"/>
    <xsl:text>}</xsl:text>
  </xsl:template>

  <xsl:template match="exp|test">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="equals|nequals|lt|lte|gt|gte">
    <xsl:apply-templates select="exp[1]"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="." mode="symbol"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="exp[2]"/>
  </xsl:template>
 
  <xsl:template match="equals" mode="symbol">
    <xsl:text>==</xsl:text>
  </xsl:template>

  <xsl:template match="nequals" mode="symbol">
    <xsl:text>!=</xsl:text>
  </xsl:template>

  <xsl:template match="lt" mode="symbol">
    <xsl:text>&lt;</xsl:text>
  </xsl:template>

  <xsl:template match="lte" mode="symbol">
    <xsl:text>&lt;=</xsl:text>
  </xsl:template>

  <xsl:template match="gt" mode="symbol">
    <xsl:text>&gt;</xsl:text>
  </xsl:template>

  <xsl:template match="gte" mode="symbol">
    <xsl:text>&gt;=</xsl:text>
  </xsl:template>

  <xsl:template match="mem">
    <xsl:text>MEMBER(</xsl:text>
    <xsl:apply-templates select="symtbl_id"/>
    <xsl:text>,yytext)</xsl:text>
  </xsl:template>

</xsl:stylesheet>
