stylesheet [version="1.0"];
output [method="text"];

include "regexp.xsl";

strip-space "*";

let newline = {"
"};

"/" -> {
apply-templates to "xlex"
};

"xlex" -> {
apply-templates to "declarations",
value-of "$newline",
"%%",
value-of "$newline",
apply-templates to "rules",
value-of "$newline"
};

"declarations" -> {
apply-templates to "inclusive_states|exclusive_states|token_declaration|abbreviation|counter_declaration"
};

"inclusive_states" -> {
"%s",
for-each "state_id" {
" ",
value-of "."
},
value-of "$newline"
};

"exclusive_states" -> {
"%s",
for-each "state_id" {
" ",
value-of "."
},
value-of "$newline"
};

"abbreviation" -> {
value-of "id",
" ",
apply-templates to "regexp",
value-of "$newline"
};

"token_declaration" -> {
"%token",
for-each "token_id" {
" ",
value-of "."
},
value-of "$newline"
};

"counter_declaration" -> {
"%counter",
for-each "counter_id" {
" ",
value-of "."
},
value-of "$newline"
};

"rules" -> {
apply-templates to "rule|scope"
};

"rule" -> {
apply-templates to "scon",
apply-templates to "regexp",
" ",
apply-templates to "action",
value-of "$newline"
};

"scon" -> {
"&lt;",
for-each "state_id" {
value-of ".",
if "position() != last()" {
","
}
},
"&gt;"
};

"any_state" -> {
"*"
};

"scope" -> {
apply-templates to "scon",
"{",
value-of "$newline",
apply-templates to "rule|scope",
"}",
value-of "$newline"
};

"action|then|else" -> {
"",
apply-templates to "./*",
""
};

"fallthru" -> {
"|"
};

"token" -> {
"TOKEN(",
value-of "token_id",
");"
};

"tokentext" -> {
"TOKENTEXT(",
value-of "token_id",
");"
};

"assign" -> {
value-of "counter_id",
" := ",
value-of "num",
";"
};

"begin" -> {
"BEGIN(",
value-of "state_id",
");"
};

"push" -> {
"PUSH(",
value-of "state_id",
");"
};

"pop" -> {
"POP;"
};

"start" -> {
"START(",
value-of "token_id",
");"
};

"end" -> {
"END;"
};

"continue" -> {
"CONTINUE;"
};

"echo" -> {
"ECHO;"
};

"skip" -> {
"SKIP;"
};

"cut" -> {
"INCR(",
apply-templates to ".",
");"
};

"trim" -> {
"INCR(",
apply-templates to ".",
");"
};

"incr" -> {
"INCR(",
value-of ".",
");"
};

"decr" -> {
"DECR(",
value-of ".",
");"
};

"error" -> {
"ERROR(&quot;",
value-of ".",
"&quot;);"
};

"if" -> {
"IF(",
apply-templates to "test",
") THEN ",
apply-templates to "then",
" ELSE ",
apply-templates to "else"
};

"regexp|test|exp" -> {
apply-templates
};

"equals" -> {
apply-templates to "*[1]",
" == ",
apply-templates to "*[2]"
};

"lt" -> {
apply-templates to "*[1]",
" &lt; ",
apply-templates to "*[2]"
};

"lte" -> {
apply-templates to "*[1]",
" &lt;= ",
apply-templates to "*[2]"
};

"nequals" -> {
apply-templates to "*[1]",
" != ",
apply-templates to "*[2]"
};

"gt" -> {
apply-templates to "*[1]",
" &gt; ",
apply-templates to "*[2]"
};

"gte" -> {
apply-templates to "*[1]",
" &gt;= ",
apply-templates to "*[2]"
};

"counter_id" -> {
value-of "."
};

"num" -> {
value-of "."
};

