#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "xmlbase.h"



int current_tok = 0;

/* TODO: Check that all strings that get put into XML trees 
   are safely allocated (won't go away until we delete them)
   What to do about allocated vs. literal strings? */

xml_content_t* ELEMENT(char* x,xml_list_t* elts) {
  xml_content_t* c = NEW(xml_content_t);
  c->data.element = NEW(xml_element_t);
  assert(c && c->data.element);
  c->type = CT_ELEMENT;
  c->data.element->name = x;
  c->data.element->attlist = NULL;
  c->data.element->content = elts;
  return c;
}

xml_content_t* ELEMENTATTS(char* x,xml_list_t* atts, xml_list_t* elts) {
  xml_content_t* c = NEW(xml_content_t);
  c->data.element = NEW(xml_element_t);
  assert(c && c->data.element);
  c->type = CT_ELEMENT;
  c->data.element->name = x;
  c->data.element->attlist = atts;
  c->data.element->content = elts;
  return c;
}

xml_list_t* APPEND(xml_list_t* l1, xml_list_t* l2) {
  if(l1 == NULL) return l2;
  else {
    l1->next = APPEND(l1->next, l2);
    return l1;
  }
}

xml_list_t* CONS(void* x, xml_list_t* xs) {
  xml_list_t* cell = NEW(xml_list_t);
  assert(cell);
  cell->data = x;
  cell->next = xs;
  return cell;
}

xml_list_t* NIL = NULL;

xml_attribute_t* ATTRIBUTE(char* name, char* value) {
  xml_attribute_t* att = NEW(xml_attribute_t);
  assert(att);
  att->name = name;
  att->value = value;
  return att;
}

xml_content_t* TEXT(char* t) {
  xml_content_t* c = NEW(xml_content_t);
  assert(c);
  c->type = CT_TEXT;
  c->data.text = t;
  return c;
}

xml_content_t* LITERAL(char* t) {
  xml_content_t* c = NEW(xml_content_t);
  assert(c);
  c->type = CT_LITERAL;
  c->data.text = t;
  return c;
}


xml_list_t* REV(xml_list_t* l) {
  xml_list_t *m = NULL, *tmp = l;
  while(l != NULL) {
    tmp = l->next;
    l->next = m;
    m = l;
    l = tmp;
  }
  return m;
}


void fprint_xml_content(FILE* fp, xml_content_t* x);
void fprint_xml_content_list(FILE* fp, xml_list_t* x);
void fprint_xml_element(FILE* fp, xml_element_t* x);
void fprint_xml_attribute_list(FILE* fp, xml_list_t* x);

void fprint_xml_string(FILE* fp, char* s) {
  while(*s != '\0') {
    switch (*s) {
    case '<':
      fprintf(fp,"&lt;");
      break;
    case '>':
      fprintf(fp,"&gt;");
      break;
    case '\'':
      fprintf(fp,"&apos;");
      break;
    case '\"':
      fprintf(fp,"&quot;");
      break;
    case '&':
      fprintf(fp,"&amp;");
      break;
    default:
      fputc(*s,fp);
      break;
    }
    s++;
  }
}

void fprint_xml_content(FILE* fp, xml_content_t* x) {
  switch(x->type) {
  case CT_ELEMENT: 
    fprint_xml_element(fp,x->data.element);
    break;
  case CT_LITERAL:
    fputs(x->data.text,fp);
    break;
  case CT_TEXT:
    fprint_xml_string(fp,x->data.text);
    break;
  }
}

void fprint_xml_content_list(FILE* fp, xml_list_t* l) {
  while(l != NIL) {
    fprint_xml_content(fp,(xml_content_t*)l->data);
    l = l->next;
  }
}

void fprint_xml_attribute_list(FILE* fp, xml_list_t* l) {
  while(l != NIL) {
    xml_attribute_t* att = (xml_attribute_t*)l->data;
    fprintf(fp," %s=\"%s\"",att->name,att->value);
    l = l->next;
  }
}


void fprint_xml_element(FILE* fp, xml_element_t* e) {
  fprintf(fp,"<%s", e->name);
  if(e->attlist != NULL) {
    fprint_xml_attribute_list(fp,e->attlist);
  }
  if(e->content == NULL) 
    fprintf(fp,"/>\n");
  else {
    fprintf(fp,">");
    fprint_xml_content_list(fp,e->content);
    fprintf(fp,"</%s>\n",e->name);
  }
}
/* Warning: This is a dangerous function.  Its argument list must be terminated by a NULL void* pointer, and all the previous args must be valid void*'s. */
xml_list_t* LIST(void* hd, ...) {
  va_list va;
  xml_list_t* l = CONS(hd,NIL);
  void* p;
  va_start(va,hd);
  while((p = va_arg(va,void*)) != NULL) {
    l = CONS(p,l);
  }
  return REV(l);
}


#define INIT_SIZE 10

void grow_symtbl(symtbl_t tbl) {
  if(tbl->limit == 0) {
    tbl->size = 0;
    tbl->limit = INIT_SIZE;
    tbl->arr = calloc(sizeof(char*),INIT_SIZE);
    assert(tbl->arr);
  } else {
    tbl->arr = realloc(tbl->arr,2*tbl->limit*sizeof(char*));
    tbl->limit = 2*tbl->limit;
  }
}

void ADD(symtbl_t tbl, char* s) {
  if (MEMBER(tbl,s)) return;
  if (tbl->size == tbl->limit) grow_symtbl(tbl);
  tbl->arr[tbl->size] = strdup(s);
  tbl->size++;
}

void REMOVE (symtbl_t tbl, char* s) {
  int i;
  for(i = 0; i < tbl->size; i++) {
    if (strcmp(tbl->arr[i],s) == 0) {
      free(tbl->arr[i]);
      tbl->arr[i] = tbl->arr[tbl->size-1];
      tbl->size--;
    }
  }
}

int MEMBER (symtbl_t tbl, char* s) {
  int i;
  for(i = 0; i < tbl->size; i++) {
    if (strcmp(tbl->arr[i],s) == 0) return 1;
  }
  return 0;
}
