#ifndef __XMLBASE_H__
#define __XMLBASE_H__

#include <stdio.h>
#include <stdlib.h>

/* XML content model stuff */
typedef struct xml_content xml_content_t;
typedef struct xml_attribute xml_attribute_t;
typedef struct xml_list xml_list_t;
typedef struct xml_element xml_element_t;

struct xml_list {
  void* data;
  xml_list_t* next;
};
enum xml_content_type {CT_ELEMENT, CT_TEXT, CT_LITERAL};
struct xml_content {
  enum xml_content_type type;
  union {
    xml_element_t* element;
    char* text;
  } data;
};

struct xml_element {
  char* name;
  xml_list_t* attlist;
  xml_list_t* content;
};

struct xml_attribute {
  char* name;
  char* value;
};

void fprint_xml_content(FILE* fp, xml_content_t*);
void fprint_xml_string(FILE* fp, char* s); 

#define NEW(t) (t*)malloc(sizeof(t))

/* Macros for lexer actions */
/* TODO: Put this in a separate file? Might merge lex&yacc eventually anyway. */
/* TODO: Import the right version for platform. This isn't ANSI C so it
   may vary or not exist. */

extern const char * token_names[];
extern int CNTR_lineno;
extern char* strdup(const char*);
extern int current_tok;

#define INCR(x)     {(x)++;}
#define ERROR(x)    {fprintf(stderr,"%d:%s\n",CNTR_lineno,x); exit(1);}
#define SKIP        {}
#define PUSH(x)     {yy_push_state(x);}
#define POP         {yy_pop_state();}
#define CUT(x)      {yytext += (x);} /* Better name? */
#define TRIM(x)     {yytext[yyleng-(x)] = '\0';}

#ifdef XML_OUT
#define CONTINUE {printf("%s",yytext);}
#define END      {printf("</%s>\n",token_names[current_tok-NONE]);}
#define TOKEN(x) {printf("<%s/>\n",token_names[(x)-NONE]);}
#define TOKENTEXT(x) {printf("<%s>%s</%s>\n",token_names[(x)-NONE],yytext,token_names[(x)-NONE]);}
#define START(x) {current_tok = x; printf("<%s>",token_names[(x)-NONE]);}
#endif

#ifdef YYLEX_OUT
#define CONTINUE {yymore();}
#define END      {yylval.xml_string = strdup(yytext); return current_tok;}
#define TOKEN(x) {return (x);}
#define TOKENTEXT(x) {yylval.xml_string = strdup(yytext); return (x);}
#define START(x) {current_tok = (x);}
#endif


/* Macros and functions for parser actions */

xml_content_t* ELEMENT(char* x,xml_list_t* elts);

/* TODO: get rid of these, once bootstrapping works */
#define ELEMENT0(name) ELEMENT(name,NIL)
#define ELEMENT1(name,child) ELEMENT(name,CONS(child,NIL))
#define ELEMENT2(name,child1,child2) ELEMENT(name,CONS(child1,CONS(child2,NIL)))
#define ELEMENT3(name,child1,child2,child3) ELEMENT(name,CONS(child1,CONS(child2,CONS(child3,NIL))))

xml_attribute_t* ATTRIBUTE(char* name, char* value);


xml_content_t* ELEMENTATTS(char* X, 
			   xml_list_t* atts, 
			   xml_list_t* elts);

xml_list_t* CONS(void*, xml_list_t*);
xml_list_t* APPEND(xml_list_t*, xml_list_t*);
 
extern xml_list_t* NIL;

xml_content_t* TEXT(char*);

xml_content_t* LITERAL(char*);

xml_list_t* REV(xml_list_t*);

#define SINGL(x) (CONS(x,NIL))

xml_list_t* LIST(void*, ...);


struct symtbl {
  int size;
  int limit;
  char** arr;
};

typedef struct symtbl* symtbl_t;
void ADD(symtbl_t, char*);
void REMOVE(symtbl_t, char*);
int MEMBER(symtbl_t, char*);


#endif
