%{
#include "xmlbase.h"
%}

%union {
xml_list_t* xml_list;
xml_element_t* xml_element;
xml_attribute_t* xml_attribute;
xml_content_t* xml_content;
char* xml_string;
};

%token NONE START_DEC TYPE_DEC TOKEN_DEC SYMTBL_DEC BAR SEMI SECTEND 
%token LIST_TY STRING_TY ELEMENT_TY ATTRIBUTE_TY CONTENT_TY
%token LBRACE RBRACE LANGLE RANGLE LPAREN RPAREN NL COLON COMMA PREC EOF_TOK 
%token EMPTY_TM CONCAT_TM ELEMENT_TM TEXT_TM ATTRIBUTE_TM
%token NIL_TM APPEND_TM CONS_TM LIST_TM REV_TM ERROR_TM VAR_TM ACCEPT_TM LITERAL_TM
%token <xml_string> ASSOC_DEC NONTERM_ID TOKEN_ID CHAR STRING_TM NUM_TM ID SYMTBL_ID
%token DOT COLONCOLON PLUSPLUS LBRACK RBRACK EQUALS ADD_TM REMOVE_TM ERROR_MATCH
%token BANG AT

%nonassoc EQUALS 
%right PLUSPLUS
%right COLONCOLON
%right DOT

%type <xml_content> goal rule seq effect action ty nonterm_id token_id symtbl_id
%type <xml_content> char match precedence term symbol
%type <xml_list> sect1 sect2 namelist1 namelist2 matches atts terms symbols seq_list
%type <xml_list> namelist3 
%%

goal : sect1 SECTEND sect2 EOF_TOK         {$$ = ELEMENT("xyacc",CONS(ELEMENT("declarations",REV($1)),CONS(ELEMENT("rules",REV($3)),NIL)));
                                            fprint_xml_content(stdout,$$);
                                            YYACCEPT;}
;

sect1 : sect1 TOKEN_DEC namelist2 NL  {$$ = CONS(ELEMENT("token_declaration",REV($3)),$1);}
      | sect1 TOKEN_DEC LANGLE ty RANGLE namelist2 NL  {$$ = CONS(ELEMENT("token_declaration",CONS(ELEMENT1("type",$4),REV($6))),$1);}
      | sect1 ASSOC_DEC namelist2 NL  {$$ = CONS(ELEMENTATTS("associativity_declaration",CONS(ATTRIBUTE("direction",$2),NIL),REV($3)),$1);}
      | sect1 TYPE_DEC LANGLE ty RANGLE namelist1 NL  {$$ = CONS(ELEMENT("type_declaration",CONS(ELEMENT1("type",$4),REV($6))),$1);}
      | sect1 START_DEC nonterm_id NL {$$ = CONS(ELEMENT("start_declaration",SINGL($3)),$1);}
      | sect1 SYMTBL_DEC namelist3 NL {$$ = CONS(ELEMENT("symtbl_declaration",REV($3)),$1);}
      | sect1 NL                   {$$ = $1;}
      |                            {$$ = NIL;}
      | error                      {ERROR("expecting section 1 declaration");}
      ;

namelist1 : namelist1 nonterm_id   {$$ = CONS ($2,$1);}
          | nonterm_id             {$$ = SINGL($1);}
          | error                  {ERROR("expecting nonterm name list");}
          ;

namelist2 : namelist2 token_id     {$$ = CONS ($2,$1);}
          | token_id               {$$ = SINGL($1);}
          | error                  {ERROR("expecting token name list");}
          ;

namelist3 : namelist3 symtbl_id     {$$ = CONS ($2,$1);}
          | symtbl_id               {$$ = SINGL($1);}
          | error                  {ERROR("expecting token name list");}
          ;

ty : STRING_TY              {$$ = ELEMENT0("stringty");}
   | ELEMENT_TY             {$$ = ELEMENT0("elementty");}
   | ATTRIBUTE_TY           {$$ = ELEMENT0("attributety");}
   | CONTENT_TY             {$$ = ELEMENT0("contentty");}
   | LIST_TY ty             {$$ = ELEMENT1("listty", $2);}
;
sect2 : sect2 rule SEMI     {$$ = CONS($2,$1);}
      |                           {$$ = NIL;}
      | error                      {ERROR("expecting section 2 declaration");}
      ;

rule : nonterm_id COLON matches {$$ = ELEMENT("rule", CONS($1, REV($3)));}
;

matches : matches BAR match {$$ = CONS($3,$1);}
        | match             {$$ = SINGL($1);}
        ;

match : symbols precedence seq {$$ = ELEMENT("match",APPEND(REV($1),CONS($2,SINGL($3))));}
      | symbols seq            {$$ = ELEMENT("match",APPEND(REV($1),SINGL($2)));}
      | ERROR_MATCH seq {$$ = ELEMENT2("match",ELEMENT0("error_match"),$2);}
      | error 	    {ERROR("expecting match");}
      ;

symbols : symbols symbol {$$ = CONS($2,$1);}
        |                {$$ = NIL;}
	| error 	    {ERROR("expecting symbols");}
        ;

char : CHAR              { $$ = ELEMENT1("char",TEXT($1)); }
;

symtbl_id : SYMTBL_ID      {$$ = ELEMENT1("symtbl_id", TEXT($1));}
;

token_id : TOKEN_ID      {$$ = ELEMENT1("token_id", TEXT($1));}
;

nonterm_id : NONTERM_ID  { $$ = ELEMENT1("nonterm_id", TEXT($1)); }
;

symbol : token_id        { $$ = ELEMENT1("symbol",$1); }
       | nonterm_id      { $$ = ELEMENT1("symbol",$1); }
       | char            { $$ = ELEMENT1("symbol",$1); }
;

precedence : PREC token_id    {$$ = ELEMENT1("precedence",$2);}
;

seq : LBRACE seq_list action RBRACE
                              {$$ = ELEMENT("seq",APPEND(REV($2),SINGL($3)));}
;

seq_list :  {$$ = NIL;}
| seq_list effect SEMI {$$ = CONS($2,$1);}
;

effect : ADD_TM LPAREN ID COMMA term RPAREN {$$ = ELEMENT2("add",ELEMENT1("symtbl_id",TEXT($3)),ELEMENT1("term",$5));}
       | REMOVE_TM LPAREN ID COMMA term RPAREN {$$ = ELEMENT2("remove",ELEMENT1("symtbl_id",TEXT($3)),ELEMENT1("term",$5));}
;

action : term {$$ = ELEMENT1("action",ELEMENT1("term",$1));}
       | ERROR_TM LPAREN term RPAREN
                         {$$ = ELEMENT1("action",ELEMENT1("error",$3));}
       | ACCEPT_TM LPAREN term RPAREN 
                         {$$ = ELEMENT1("action",ELEMENT1("accept",$3));}
;

/* lame syntax for now... */
term : EMPTY_TM             {$$ = ELEMENT0("empty");}
     | CONCAT_TM LPAREN term COMMA term RPAREN 
                         {$$ = ELEMENT2("concat",$3,$5);}
     | term DOT term 
                         {$$ = ELEMENT2("concat",$1,$3);}
     | STRING_TM           {$$ = ELEMENT1("string",TEXT($1));}
     | ELEMENT_TM LPAREN term COMMA term COMMA term RPAREN
                         {$$ = ELEMENT3("element",ELEMENT1("name",$3),ELEMENT1("attlist",$5),ELEMENT1("content",$7));}
     | ID                {$$ = ELEMENT3("element",ELEMENT1("name",ELEMENT1("string",TEXT($1))),ELEMENT1("attlist",ELEMENT0("nil")),ELEMENT1("content",ELEMENT0("nil")));}
     | ID LBRACK terms RBRACK {$$ = ELEMENT3("element",ELEMENT1("name",ELEMENT1("string",TEXT($1))),ELEMENT1("attlist",ELEMENT0("nil")),ELEMENT1("content",ELEMENT("list",REV($3))));}
     | ID LANGLE atts RANGLE 
                         {$$ = ELEMENT3("element",ELEMENT1("name",ELEMENT1("string",TEXT($1))), ELEMENT1("attlist",ELEMENT("list",REV($3))),ELEMENT1("content",ELEMENT0("nil")));}
     | ID  LANGLE atts RANGLE LBRACK terms RBRACK {$$ = ELEMENT3("element",ELEMENT1("name",ELEMENT1("string",TEXT($1))),ELEMENT1("attlist",ELEMENT("list",REV($3))),ELEMENT1("content",ELEMENT("list",REV($6))));}
     | BANG ID LANGLE term RANGLE LBRACK term RBRACK { $$ = ELEMENT3("element",ELEMENT1("name",ELEMENT1("string",TEXT($2))),ELEMENT1("attlist",$4),ELEMENT1("content",$7)); }
     | TEXT_TM LPAREN term RPAREN
                         {$$ = ELEMENT1("text",$3);}
     | LITERAL_TM LPAREN term RPAREN
                         {$$ = ELEMENT1("literal",$3);}
     | ATTRIBUTE_TM LPAREN term COMMA term RPAREN 
                         {$$ = ELEMENT2("attribute",ELEMENT1("name",$3),ELEMENT1("value",$5));}
     | AT ID EQUALS term 
                         {$$ = ELEMENT2("attribute",ELEMENT1("name",TEXT($2)),ELEMENT1("value",$4));}
     | NIL_TM           {$$ = ELEMENT0("nil");}
     | APPEND_TM LPAREN term COMMA term RPAREN
                         {$$ = ELEMENT2("append", $3, $5);}
     | term PLUSPLUS term
                         {$$ = ELEMENT2("append", $1, $3);}
     | CONS_TM LPAREN term COMMA term RPAREN
                         {$$ = ELEMENT2("cons", $3, $5);}
     | term COLONCOLON term 
			 {$$ = ELEMENT2("cons",$1,$3);}
     | LIST_TM LPAREN terms RPAREN
                         {$$ = ELEMENT("list",REV($3));}
     | LBRACK terms RBRACK 
			 {$$ = ELEMENT("list",REV($2));}
     | LBRACK terms BAR term RBRACK 
			 {$$ = ELEMENT2("append",ELEMENT("list",REV($2)),$4);}
     | LBRACK RBRACK 
			 {$$ = ELEMENT0("nil");}
     | REV_TM LPAREN term RPAREN 
                         {$$ = ELEMENT1("rev",$3);}
     | VAR_TM NUM_TM     {$$ = ELEMENTATTS("var",CONS(ATTRIBUTE("num",$2),NIL),NIL);}
     | LPAREN term RPAREN {$$ = $2;}
     | error             { ERROR ("expecting term");}
;

terms : terms COMMA term { $$ = CONS($3,$1);}
      | term             { $$ = SINGL($1);}
;
atts : atts COMMA ID EQUALS term { $$ = CONS(ELEMENT2("attribute",ELEMENT1("name",ELEMENT1("string",TEXT($3))),ELEMENT1("value",$5)),$1);}
     | ID EQUALS term             { $$ = SINGL(ELEMENT2("attribute",ELEMENT1("name",ELEMENT1("string",TEXT($1))),ELEMENT1("value",$3)));}
;
%%
