<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="xml" indent="yes"/>

  <xsl:strip-space elements="*"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>


  <xsl:template match="/">
    <xsl:element name="yacc">
      <xsl:apply-templates select="xyacc"/>
    </xsl:element>
  </xsl:template>

<!-- TODO: Make sure NONE is a special symbol -->

  <xsl:template match="xyacc">
    <xsl:element name="declarations">
      <token_declaration><token_id>NONE</token_id></token_declaration>
      <code_declaration>
<ccode>
#include "xmlbase.h"
</ccode>
      </code_declaration>
      <union_declaration>
        <union_field><type>xml_list_t*</type>foo<type_id>xml_list</type_id></union_field>
        <union_field><type>xml_element_t*</type><type_id>xml_element</type_id></union_field>
        <union_field><type>xml_attribute_t*</type><type_id>xml_attribute</type_id></union_field>
        <union_field><type>xml_content_t*</type><type_id>xml_content</type_id></union_field>
        <union_field><type>char*</type><type_id>xml_string</type_id></union_field>
      </union_declaration>
      <xsl:call-template name="symtbl_definitions"/>
      <xsl:apply-templates select="declarations"/>
    </xsl:element>
    <xsl:element name="rules">
      <xsl:apply-templates select="rules"/>
    </xsl:element>
    <xsl:element name="definitions"/>
  </xsl:template>

  <xsl:template name="symtbl_definitions">
    <xsl:element name="code_declaration">
      <xsl:for-each select="/xyacc/declarations/symtbl_declaration/symtbl_id">
        <xsl:text>extern symtbl_t </xsl:text>
        <xsl:apply-templates select="."/>
	<xsl:text>;</xsl:text>
	<xsl:value-of select="$newline"/>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>


<!-- Declarations -->

  <xsl:template match="declarations">
    <xsl:apply-templates select="token_declaration|associativity_declaration|type_declaration|start_declaration"/>
  </xsl:template>

  <xsl:template match="symtbl_declaration">
    <!-- do nothing -->
  </xsl:template>

  <xsl:template match="token_declaration">
    <xsl:element name="token_declaration">
      <xsl:for-each select="type">
        <xsl:element name="type_id">
          <xsl:apply-templates select="."/>
        </xsl:element>
      </xsl:for-each>
      <xsl:for-each select="token_id">
        <xsl:apply-templates select="."/>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>

   <xsl:template match="associativity_declaration">
     <xsl:element name="associativity_declaration">
      <xsl:copy-of select="./@direction"/>
      <xsl:for-each select="token_id">
        <xsl:apply-templates select="."/>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>

   <xsl:template match="type_declaration">
    <xsl:element name="type_declaration">
      <xsl:element name="type_id">
        <xsl:apply-templates select="type"/>
      </xsl:element>
      <xsl:for-each select="nonterm_id">
         <xsl:copy-of select="."/>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>
 
  <xsl:template match="start_declaration">
     <xsl:copy-of select="."/>
  </xsl:template>

<!-- Types -->

  <xsl:template match="type">
    <xsl:apply-templates select="stringty|elementty|contentty|attributety|listty"/>
  </xsl:template>

  <xsl:template match="stringty">
    <xsl:text>xml_string</xsl:text>
  </xsl:template>

  <xsl:template match="elementty">
    <xsl:text>xml_element</xsl:text>
  </xsl:template>

  <xsl:template match="attributety">
    <xsl:text>xml_attribute</xsl:text>
  </xsl:template>

  <xsl:template match="contentty">
    <xsl:text>xml_content</xsl:text>
  </xsl:template>

  <xsl:template match="listty">
    <xsl:text>xml_list</xsl:text>
  </xsl:template>

<!-- Rules -->

  <xsl:template match="rules">
    <xsl:apply-templates select="rule"/>
  </xsl:template>

  <xsl:template match="rule">
    <xsl:element name="rule">
    <xsl:copy-of select="nonterm_id"/>
    <xsl:for-each select="match">
      <xsl:element name="match">
      <xsl:apply-templates select="."/>
      </xsl:element>
    </xsl:for-each>
    </xsl:element>
  </xsl:template>

  <xsl:template match="error_match">
    <xsl:element name="error_match"/>
  </xsl:template>

  <xsl:template match="symbol">
    <xsl:element name="symbol">
    <xsl:apply-templates select="token_id|nonterm_id|char"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="token_id">
     <xsl:element name="token_id">
       <xsl:text>TOK_</xsl:text>
       <xsl:value-of select="."/>
     </xsl:element>
  </xsl:template>

  <xsl:template match="symtbl_id">
     <xsl:element name="symtbl_id">
       <xsl:text>SYMTBL_</xsl:text>
       <xsl:value-of select="."/>
     </xsl:element>
  </xsl:template>

  <xsl:template match="nonterm_id">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="char">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="precedence">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="seq">
    <xsl:element name="action">
    <xsl:element name="ccode">
    <xsl:for-each select="*">
      <xsl:apply-templates select="."/>
    </xsl:for-each>
    </xsl:element>
    </xsl:element>
  </xsl:template>

  <xsl:template match="action">
      <xsl:for-each select="term">
          <xsl:text>$$ = </xsl:text>
          <xsl:apply-templates select="."/>
  	  <xsl:text>;</xsl:text>
      </xsl:for-each>
      <xsl:for-each select="error">
          <xsl:text>ERROR(</xsl:text>
          <xsl:apply-templates select="*[1]"/>
          <xsl:text>);</xsl:text>
      </xsl:for-each>
      <xsl:for-each select="accept">
	  <xsl:text>$$ = </xsl:text>
	  <xsl:apply-templates select="."/>
	  <xsl:text>;</xsl:text>
	  <xsl:value-of select="$newline"/>
	  <xsl:text>fprint_xml_content(stdout,$$);</xsl:text>
          <xsl:value-of select="$newline"/>
          <xsl:text>YYACCEPT;</xsl:text>
      </xsl:for-each>
  </xsl:template>

<!-- Actions -->

  <xsl:template match="empty">
    <xsl:text>""</xsl:text>
  </xsl:template>

  <xsl:template match="concat">
    <xsl:text>CONCAT(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="string">
    <xsl:text>"</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>"</xsl:text>
  </xsl:template>


  <xsl:template match="literal">
    <xsl:text>LITERAL(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="text">
    <xsl:text>TEXT(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="element">
    <xsl:text>ELEMENTATTS(</xsl:text>
    <xsl:apply-templates select="name"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="attlist"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="content"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="attribute">
    <xsl:text>ATTRIBUTE(</xsl:text>
    <xsl:apply-templates select="name"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="value"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="nil">
    <xsl:text>NIL</xsl:text>
  </xsl:template>

  <xsl:template match="cons">
    <xsl:text>CONS(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="append">
    <xsl:text>APPEND(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>
    
  <xsl:template match="list">
    <xsl:text>LIST(</xsl:text>
    <xsl:for-each select="./*">
      <xsl:apply-templates select="."/>
      <xsl:if test="position() != last()">
        <xsl:text>, </xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>,NIL)</xsl:text>
  </xsl:template>

  <xsl:template match="rev">
    <xsl:text>REV(</xsl:text>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>)</xsl:text>
  </xsl:template>
 
  <xsl:template match="var">
    <xsl:text>$</xsl:text>
    <xsl:value-of select="@num"/>
  </xsl:template>


  <xsl:template match="add">
    <xsl:text>ADD(</xsl:text>
    <xsl:apply-templates select="symtbl_id"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="term"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

  <xsl:template match="remove">
    <xsl:text>REMOVE(</xsl:text>
    <xsl:apply-templates select="symtbl_id"/>
    <xsl:text>,</xsl:text>
    <xsl:apply-templates select="term"/>
    <xsl:text>);</xsl:text>
  </xsl:template>

</xsl:stylesheet>
